//=========================================================================//
//-------------------------------------------------------------------------//
// $Id: ds1621.cpp,v 1.5 2003/07/20 18:07:23 lanconel Exp $                                                                    //
//-------------------------------------------------------------------------//
//  Copyright (C) 1998-2003  Claudio Lanconelli                            //
//                                                                         //
//  e-mail: lancos@libero.it                                               //
//  http://www.LancOS.com                                                  //
//-------------------------------------------------------------------------//
//                                                                         //
// This program is free software; you can redistribute it and/or           //
// modify it under the terms of the GNU  General Public License            //
// as published by the Free Software Foundation; either version2 of        //
// the License, or (at your option) any later version.                     //
//                                                                         //
// This program is distributed in the hope that it will be useful,         //
// but WITHOUT ANY WARRANTY; without even the implied warranty of          //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       //
// General Public License for more details.                                //
//                                                                         //
// You should have received a copy of the GNU  General Public License      //
// along with this program (see COPYING);     if not, write to the         //
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. //
//                                                                         //
//-------------------------------------------------------------------------//
//=========================================================================//

#include "ds1621.h"
#include "errcode.h"

/** Command structure
 * STANDALONE COMMAND (CMD_STARTCONVERSION, CMD_STOPCONVERSION)
 * Start + Address(W) + Command + Stop
 *
 * WRITE A SINGLE-BYTE REGISTER (CONFIGURATION)                                  * Start + Address(W) + Command + Data + Stop
 *
 * WRITE TO A TWO-BYTE REGISTER (TH, TL)
 * Start + Address(W) + Command + MSB Data + LSB Data + Stop
 *
 * READ FROM A SINGLE-BYTE REGISTER (CONFIGURATION, SLOPE, COUNTER)
 * Start + Address(W) + Command + Start + Address(R) + Data + Stop
 *
 * READ FROM A TWO-BYTE REGISTER (TH, TL, TEMPERATURE)
 * Start + Address(W) + Command + Start + Address(R) + MSB Data + LSB Data + Stop
 */

DS1621::DS1621(int i2c_delay) :
	CmdStartConversion(0xEE),
	CmdStopConversion(0x22),
	CmdAccessConfig(0xAC),
	CmdAccessTH(0xA1),
	CmdAccessTL(0xA2),
	CmdReadTemp(0xAA),
	CmdReadCounter(0xA8),
	CmdReadSlope(0xA9)
{
	i2c_address = BASE_ADDRESS;
	error_code = 0;
	i2c.SetDelay(i2c_delay);
}

DS1621::~DS1621()
{
}

void DS1621::Init(bool one_shot)
{
	UBYTE val = (1<<RES_BIT);

	if (one_shot)
		val |= (1<<ONESHOT_BIT);

	WriteConfiguration(val);	//delay to write eeprom
	i2c.WaitMsec(50);
}

void DS1621::Open(int port)
{
	i2c.Open(port);
}

void DS1621::Close()
{
	i2c.Close();
}

int DS1621::StartConversion()
{
	write_command(CmdStartConversion);

	return GetError();
}

int DS1621::StopConversion()
{
	write_command(CmdStopConversion);

	return GetError();
}

int DS1621::ReadConfiguration(UBYTE &val)
{
	val = read_byte_register(CmdAccessConfig);

	return GetError();
}

int DS1621::WriteConfiguration(UBYTE val)
{
	write_byte_register(CmdAccessConfig, val);

	return GetError();
}

int DS1621::ReadHighThreshold(short &val)
{
	val = read_sword_register(CmdAccessTH);

	return GetError();
}

int DS1621::WriteHighThreshold(short val)
{
	write_sword_register(CmdAccessTH, val);

	return GetError();
}

int DS1621::ReadLowThreshold(short &val)
{
	val = read_sword_register(CmdAccessTL);

	return GetError();
}

int DS1621::WriteLowThreshold(short val)
{
	write_sword_register(CmdAccessTL, val);

	return GetError();
}

int DS1621::ReadTemperature(short &val)
{
	val = read_sword_register(CmdReadTemp);
	if (GetError())
		val = 0;

	return GetError();
}

int DS1621::ReadCounter(UBYTE &val)
{
	val = read_byte_register(CmdReadCounter);
	if (GetError())
		val = 0;

	return GetError();
}

int DS1621::ReadSlope(UBYTE &val)
{
	val = read_byte_register(CmdReadSlope);
	if (GetError())
		val = 0;

	return GetError();
}

int DS1621::WaitConversion()
{
	long k;
	UBYTE val;
	int rv;

	k = 2000;
	do {
		rv = ReadConfiguration(val);
		k--;
	} while (k > 0 && rv == 0 && !(val & (1<<DONE_BIT)));

	if (k > 0)
		return rv;
	else
		return E2P_TIMEOUT;
}

/*---------------------------------------------*/

signed short DS1621::read_sword_register(UBYTE cmd_code)
{
	i2c_buffer[0] = cmd_code;

	i2c_read_packet(2);

	return ((signed short)i2c_buffer[0] << 8) | i2c_buffer[1];
}

UWORD DS1621::read_word_register(UBYTE cmd_code)
{
	i2c_buffer[0] = cmd_code;

	i2c_read_packet(2);

	return ((UWORD)i2c_buffer[0] << 8) | i2c_buffer[1];
}

signed char DS1621::read_sbyte_register(UBYTE cmd_code)
{
	i2c_buffer[0] = cmd_code;

	i2c_read_packet(1);

	return (signed char)i2c_buffer[0];
}

UBYTE DS1621::read_byte_register(UBYTE cmd_code)
{
	i2c_buffer[0] = cmd_code;

	i2c_read_packet(1);

	return (UBYTE)i2c_buffer[0];
}

void DS1621::write_sword_register(UBYTE cmd_code, signed short val)
{
	unsigned int j = 0;
	i2c_buffer[j++] = cmd_code;
	i2c_buffer[j++] = (UBYTE)(val >> 8);
	i2c_buffer[j++] = (UBYTE)val;

	i2c_write_packet(j);
}

void DS1621::write_word_register(UBYTE cmd_code, UWORD val)
{
	unsigned int j = 0;
	i2c_buffer[j++] = cmd_code;
	i2c_buffer[j++] = (UBYTE)(val >> 8);
	i2c_buffer[j++] = (UBYTE)val;

	i2c_write_packet(j);
}

void DS1621::write_sbyte_register(UBYTE cmd_code, signed char val)
{
	unsigned int j = 0;
	i2c_buffer[j++] = cmd_code;
	i2c_buffer[j++] = (UBYTE)val;

	i2c_write_packet(j);
}

void DS1621::write_byte_register(UBYTE cmd_code, UBYTE val)
{
	unsigned int j = 0;
	i2c_buffer[j++] = cmd_code;
	i2c_buffer[j++] = val;

	i2c_write_packet(j);
}

void DS1621::write_command(UBYTE cmd_code)
{
	unsigned int j = 0;
	i2c_buffer[j++] = cmd_code;

	i2c_write_packet(j);
}

/* Read packet: Start + Address(W) + Command + RepStart + Address(R) + N Bytes + Stop
 */
void DS1621::i2c_read_packet(unsigned int len)
{
	error_code = 0;
	if ( i2c.StartWrite(i2c_address, i2c_buffer, 1) != 1 )
	{
		error_code = i2c.GetErrorNumber();
		i2c.Stop();
	}
	else
	if ( i2c.Read(i2c_address, i2c_buffer, len) != len )
	{
		error_code = i2c.GetErrorNumber();
	}
}

/* Write packet: Start + Address(W) + N Bytes + Stop */
void DS1621::i2c_write_packet(unsigned int len)
{
	error_code = 0;
	if ( i2c.Write(i2c_address, i2c_buffer, len) != len )
		error_code = i2c.GetErrorNumber();
}

